@@ -80,7 +80,7 @@ def features_0D(data, marker='o', cmap='bwr', color=None, configs=None, **kwargs
     _catch_unknown_kwargs(kwargs)
     kw = _process_configs(configs, w, h)
     if data.ndim != 2:
-        raise Exception("`data` must be 2D")
+        raise Exception("`data` must be 2D or 3D (got ndim=%s)" % data.ndim)
 
     if color is None:
         cmap = cm.get_cmap(cmap)
@@ -182,7 +182,7 @@ def features_1D(data, n_rows=None, annotations='auto', share_xy=(1, 1),
 
     w, h          = kwargs.get('w', 1), kwargs.get('h', 1)
     show_borders  = kwargs.get('show_borders', True)
-    show_xy_ticks = kwargs.get('show_xy_ticks', [1, 1])
+    show_xy_ticks = kwargs.get('show_xy_ticks', (1, 1))
     title_mode    = kwargs.get('title_mode', 'outputs')
     show_y_zero   = kwargs.get('show_y_zero', False)
     tight         = kwargs.get('tight', False)
@@ -269,7 +269,7 @@ def features_1D(data, n_rows=None, annotations='auto', share_xy=(1, 1),
 
     def _process_data(data):
         if data.ndim not in (2, 3):
-            raise Exception("`data` must be 2D or 3D")
+            raise Exception("`data` must be 2D or 3D (got ndim=%s)" % data.ndim)
         if data.ndim == 2 and subplot_samples:
             print(NOTE, "`subplot_samples` w/ 2D `data` will only change "
                   "title shown, and assumes `data` dims (timesteps, samples)")
@@ -293,6 +293,8 @@ def features_1D(data, n_rows=None, annotations='auto', share_xy=(1, 1),
 
     kw = _process_configs(configs, w, h, tight, share_xy)
     _catch_unknown_kwargs(kwargs)
+    if isinstance(show_xy_ticks, (int, bool)):
+        show_xy_ticks = (show_xy_ticks, show_xy_ticks)
     data = _process_data(data)
     n_rows, n_cols, color, annotations = _get_style_info(data, n_rows, color,
                                                          annotations)
@@ -327,7 +329,7 @@ def features_1D(data, n_rows=None, annotations='auto', share_xy=(1, 1),
 
 
 def features_2D(data, n_rows=None, norm=None, cmap='bwr', reflect_half=False,
-                timesteps_xaxis=True, max_timesteps=None, share_xy=(1, 1),
+                timesteps_xaxis=False, max_timesteps=None, share_xy=(1, 1),
                 configs=None, **kwargs):
     """Plots 2D heatmaps in a standalone graph or subplot grid.
 
@@ -386,6 +388,7 @@ def features_2D(data, n_rows=None, norm=None, cmap='bwr', reflect_half=False,
               -1 --> (samples,  timesteps, channels)
               0  --> (channels, timesteps, samples)
         borderwidth: float / None. Width of subplot borders.
+        bordercolor: str / None. Color of subplot borders. Default black.
         savepath: str/None. Path to save resulting figure to. Also see `configs`.
                If None, doesn't save.
 
@@ -395,12 +398,13 @@ def features_2D(data, n_rows=None, norm=None, cmap='bwr', reflect_half=False,
 
     w, h          = kwargs.get('w', 1), kwargs.get('h', 1)
     show_borders  = kwargs.get('show_borders', True)
-    show_xy_ticks = kwargs.get('show_xy_ticks', [1, 1])
+    show_xy_ticks = kwargs.get('show_xy_ticks', (1, 1))
     show_colorbar = kwargs.get('show_colorbar', False)
     title_mode    = kwargs.get('title_mode', 'outputs')
     tight         = kwargs.get('tight', False)
     channel_axis  = kwargs.get('channel_axis', -1)
     borderwidth   = kwargs.get('borderwidth', None)
+    bordercolor   = kwargs.get('bordercolor', None)
     savepath      = kwargs.get('savepath', None)
 
     def _process_configs(configs, w, h, tight, share_xy):
@@ -435,7 +439,8 @@ def features_2D(data, n_rows=None, norm=None, cmap='bwr', reflect_half=False,
     def _catch_unknown_kwargs(kwargs):
         allowed_kwargs = ('w', 'h', 'show_borders', 'show_xy_ticks',
                           'show_colorbar', 'title_mode', 'tight',
-                          'channel_axis', 'borderwidth', 'savepath')
+                          'channel_axis', 'borderwidth', 'bordercolor',
+                          'savepath')
         for kwarg in kwargs:
             if kwarg not in allowed_kwargs:
                 raise Exception("unknown kwarg `%s`" % kwarg)
@@ -474,7 +479,7 @@ def features_2D(data, n_rows=None, norm=None, cmap='bwr', reflect_half=False,
     def _process_data(data, max_timesteps, reflect_half,
                       timesteps_xaxis, channel_axis):
         if data.ndim not in (2, 3):
-            raise Exception("`data` must be 2D or 3D")
+            raise Exception("`data` must be 2D or 3D (got ndim=%s)" % data.ndim)
 
         if max_timesteps is not None:
             data = data[..., :max_timesteps, :]
@@ -484,9 +489,10 @@ def features_2D(data, n_rows=None, norm=None, cmap='bwr', reflect_half=False,
             half_chs = data.shape[-1] // 2
             data[..., half_chs:] = np.flip(data[..., half_chs:], axis=0)
 
+        if data.ndim != 3:
+            # (1, width, height) -> one image
+            data = np.expand_dims(data, 0)
         if timesteps_xaxis:
-            if data.ndim != 3:
-                data = np.expand_dims(data, 0)
             data = np.transpose(data, (0, 2, 1))
         return data
 
@@ -501,6 +507,8 @@ def features_2D(data, n_rows=None, norm=None, cmap='bwr', reflect_half=False,
 
     _catch_unknown_kwargs(kwargs)
     kw = _process_configs(configs, w, h, tight, share_xy)
+    if isinstance(show_xy_ticks, (int, bool)):
+        show_xy_ticks = (show_xy_ticks, show_xy_ticks)
     data = _process_data(data, max_timesteps, reflect_half,
                          timesteps_xaxis, channel_axis)
     n_rows, n_cols, vmin, vmax = _get_style_info(data, n_rows, norm)
@@ -521,9 +529,13 @@ def features_2D(data, n_rows=None, norm=None, cmap='bwr', reflect_half=False,
         fig.colorbar(img, ax=axes.ravel().tolist(), **kw['colorbar'])
     if tight:
         fig.subplots_adjust(**kw['tight'])
-    if borderwidth is not None:
+    if borderwidth is not None or bordercolor is not None:
         for ax in axes.flat:
-            [s.set_linewidth(borderwidth) for s in ax.spines.values()]
+            for s in ax.spines.values():
+                if borderwidth is not None:
+                    s.set_linewidth(borderwidth)
+                if bordercolor is not None:
+                    s.set_color(bordercolor)
 
     plt.show()
     if savepath is not None:
@@ -569,10 +581,11 @@ def features_hist(data, n_rows='vertical', bins=100, xlims=None, tight=True,
                                   'row' -> limits shared along rows.
                                   Overrides 'sharex' in `configs['subplot']`.
               - sharey: bool/str. `sharex`, but for y-axis.
-        pad_xticks: bool/int. int-> kwarg to ax.tick_params(pad=). Pad xticks
-                    (<0 for above x-axis), and include only min/max shifted
-                    10% inward. Useful for `tight`. If False/0, makes no change.
-                    If None and bool(`tight` and not `sharex`), defaults to -20.
+        pad_xticks: bool / None. True: display only min/max xticks, and
+                    shifted above xaxis and 10% inward. False: no change in
+                    shown ticks. None: if bool(`tight` an not `sharex`) ->
+                    sets to True. Padding behavior configurable via
+                    configs['pad_xticks'] (see below).
         annotations: str list/'auto'/None.
             'auto': annotate each subplot with its index.
              list of str: annotate by indexing into the list.
@@ -588,6 +601,9 @@ def features_hist(data, n_rows='vertical', bins=100, xlims=None, tight=True,
             'tight':   passed to fig.subplots_adjust()
             'annot':   passed to ax.annotate()
             'save':    passed to fig.savefig() if `savepath` is not None.
+            'pad_xticks': passed to ax.annotate(). Is nested dict:
+                {'min': kw1, 'max': kw2}, applied separately for
+                ax.annotate(xmin, **kw1) and ax.annotate(xmax, **kw2).
 
     kwargs:
         w: float. Scale width  of resulting plot by a factor.
@@ -606,7 +622,7 @@ def features_hist(data, n_rows='vertical', bins=100, xlims=None, tight=True,
     """
     w, h          = kwargs.get('w', 1), kwargs.get('h', 1)
     show_borders  = kwargs.get('show_borders', True)
-    show_xy_ticks = kwargs.get('show_xy_ticks', [1, 1])
+    show_xy_ticks = kwargs.get('show_xy_ticks', (1, 1))
     title         = kwargs.get('title', None)
     borderwidth   = kwargs.get('borderwidth', None)
     savepath      = kwargs.get('savepath', None)
@@ -631,6 +647,10 @@ def features_hist(data, n_rows='vertical', bins=100, xlims=None, tight=True,
             'tight':   dict(left=0, right=1, bottom=0, top=1, wspace=0, hspace=0),
             'annot':   dict(weight='bold', fontsize=14, xy=(.02, .7),
                             xycoords='axes fraction', color='g'),
+            'pad_xticks': {'min': dict(fontsize=12, xy=(.05, .1),
+                                       xycoords='axes fraction'),
+                           'max': dict(fontsize=12, xy=(.9, .1),
+                                       xycoords='axes fraction')},
             'save': dict(),
             }
         # deepcopy configs, and override defaults dicts or dict values
@@ -644,9 +664,9 @@ def features_hist(data, n_rows='vertical', bins=100, xlims=None, tight=True,
     def _process_pad_xticks(pad_xticks, kw):
         if pad_xticks is None:
             if tight and not kw['subplot']['sharex']:
-                pad_xticks = -20
+                pad_xticks = True
             else:
-                pad_xticks = 0
+                pad_xticks = False
         return pad_xticks
 
     def _catch_unknown_kwargs(kwargs):
@@ -687,13 +707,13 @@ def features_hist(data, n_rows='vertical', bins=100, xlims=None, tight=True,
             ax.set_xlim(*xlims)
         if pad_xticks:
             xmin, xmax = ax.get_xlim()
-            xmin += .1 * abs(xmin)
-            xmax -= .1 * abs(xmax)
-            ax.set_xticks([xmin, xmax])
-            ax.tick_params(axis='x', pad=pad_xticks)
+            ax.annotate(xmin, **kw['pad_xticks']['min'])
+            ax.annotate(xmax, **kw['pad_xticks']['max'])
 
     _catch_unknown_kwargs(kwargs)
     kw = _process_configs(configs, w, h, tight, share_xy)
+    if isinstance(show_xy_ticks, (int, bool)):
+        show_xy_ticks = (show_xy_ticks, show_xy_ticks)
     pad_xticks = _process_pad_xticks(pad_xticks, kw)
 
     n_rows, n_cols, annotations = _get_style_info(data, n_rows, annotations)
@@ -756,10 +776,11 @@ def features_hist_v2(data, colnames=None, bins=100, xlims=None, ylim=None,
                                   'row' -> limits shared along rows.
                                   Overrides 'sharex' in `configs['subplot']`.
               - sharey: bool/str. `sharex`, but for y-axis.
-        pad_xticks: bool/int. int-> kwarg to ax.tick_params(pad=). Pad xticks
-                    (<0 for above x-axis), and include only min/max shifted
-                    10% inward. Useful for `tight`. If False/0, makes no change.
-                    If None and bool(`tight` and not `sharex`), defaults to -20.
+        pad_xticks: bool / None. True: display only min/max xticks, and
+                    shifted above xaxis and 10% inward. False: no change in
+                    shown ticks. None: if bool(`tight` an not `sharex`) ->
+                    sets to True. Padding behavior configurable via
+                    configs['pad_xticks'] (see below).
         side_annot: str. Text to display to the right side of rightmost subplot
               boxes, enumerated by row number ({side_annot}{row})
         configs: dict. kwargs to customize various plot schemes:
@@ -773,6 +794,9 @@ def features_hist_v2(data, colnames=None, bins=100, xlims=None, ylim=None,
             'colnames':   passed to ax.set_title()
             'side_annot': passed to ax.annotate()
             'save':       passed to fig.savefig() if `savepath` is not None.
+            'pad_xticks': passed to ax.annotate(). Is nested dict:
+                {'min': kw1, 'max': kw2}, applied separately for
+                ax.annotate(xmin, **kw1) and ax.annotate(xmax, **kw2).
 
     kwargs:
         w: float. Scale width  of resulting plot by a factor.
@@ -791,7 +815,7 @@ def features_hist_v2(data, colnames=None, bins=100, xlims=None, ylim=None,
     """
     w, h          = kwargs.get('w', 1), kwargs.get('h', 1)
     show_borders  = kwargs.get('show_borders', True)
-    show_xy_ticks = kwargs.get('show_xy_ticks', [1, 1])
+    show_xy_ticks = kwargs.get('show_xy_ticks', (1, 1))
     title         = kwargs.get('title', None)
     borderwidth   = kwargs.get('borderwidth', None)
     savepath      = kwargs.get('savepath', None)
@@ -818,6 +842,10 @@ def features_hist_v2(data, colnames=None, bins=100, xlims=None, ylim=None,
             'colnames':   dict(weight='bold', fontsize=14),
             'side_annot': dict(weight='bold', fontsize=14,
                                xy=(1.02, .5), xycoords='axes fraction'),
+            'pad_xticks': {'min': dict(fontsize=12, xy=(.05, .1),
+                                       xycoords='axes fraction'),
+                           'max': dict(fontsize=12, xy=(.9, .1),
+                                       xycoords='axes fraction')},
             'save': dict(),
             }
         # deepcopy configs, and override defaults dicts or dict values
@@ -831,9 +859,9 @@ def features_hist_v2(data, colnames=None, bins=100, xlims=None, ylim=None,
     def _process_pad_xticks(pad_xticks, kw):
         if pad_xticks is None:
             if tight and not kw['subplot']['sharex']:
-                pad_xticks = -20
+                pad_xticks = True
             else:
-                pad_xticks = 0
+                pad_xticks = False
         return pad_xticks
 
     def _catch_unknown_kwargs(kwargs):
@@ -872,13 +900,13 @@ def features_hist_v2(data, colnames=None, bins=100, xlims=None, ylim=None,
             ax.set_xlim(*xlims)
         if pad_xticks:
             xmin, xmax = ax.get_xlim()
-            xmin += .1 * abs(xmin)
-            xmax -= .1 * abs(xmax)
-            ax.set_xticks([xmin, xmax])
-            ax.tick_params(axis='x', pad=pad_xticks)
+            ax.annotate(xmin, **kw['pad_xticks']['min'])
+            ax.annotate(xmax, **kw['pad_xticks']['max'])
 
     _catch_unknown_kwargs(kwargs)
     kw = _process_configs(configs, w, h, share_xy)
+    if isinstance(show_xy_ticks, (int, bool)):
+        show_xy_ticks = (show_xy_ticks, show_xy_ticks)
     pad_xticks = _process_pad_xticks(pad_xticks, kw)
 
     n_rows, n_cols = _get_data_info(data)
